@@ -91,8 +91,8 @@ kmgw.calc<-function(time,status,keepCens=TRUE){
         } else if (dstat=="0"){
             ci<-tab[,1]
             di<-rep(0,length(tab))
-        }
-    } else stop("status should be 0 or 1")
+        } else stop("status should be 0 or 1") # fix 19-Feb-2025
+    } 
     
     y<-as.numeric(dimnames(tab)[[1]])
     k<-length(y)
@@ -121,6 +121,7 @@ kmgw.calc<-function(time,status,keepCens=TRUE){
     ##     ni[i]=number at risk just before time[i]
     ##     KM[i]=Kaplan-Meier at time[i]
     ##     gw[i]=Greenwood variance at time[i]
+    
     out<-list(time=y[I],ci=ci[I],di=di[I],ni=ni[I],
         KM=KM[I],gw=gw[I])
     out
@@ -196,7 +197,7 @@ getmarks.x<-function(x){
     x$time[x$ci>0]
 }
 
-borkowf.calc<-function(x,type="log",alpha=.05){
+borkowf.calc<-function(x,type="log",alpha=.05, twosamp_output=FALSE){
     ## calculate the Borkowf CIs
     ##    type="log" ... do log transformation
     ##    type="logs"...log transformation with shrinkage
@@ -321,14 +322,24 @@ borkowf.calc<-function(x,type="log",alpha=.05){
     #    upper=upper,row.names=Interval)
     #
     # getmarks.x gets censoring marks for plotting
+    
+    if(twosamp_output){
+      # parameters to use in two-sample case
+      out <- list(KM=KM,KMs=KMs,VarH=VarH,VarHs=VarHs)
+      
+    } else{
     out<-list(cens=getmarks.x(x),L=L,Lin=Lin,R=R,Rin=Rin,
         surv=surv,lower=lower,upper=upper,conf.level=1-alpha)
+    }
+    
     out
 }
 
 kmciBorkowf<-function(time,status,type="log",alpha=0.05){
     x<-kmgw.calc(time,status,keepCens=TRUE)
     out<-borkowf.calc(x,type,alpha)
+    out$num <- length(time)
+    out$events <- length(which(status == max(status)))
     class(out)<-"kmciLR"
     out
 }
@@ -400,6 +411,277 @@ kmciTG<-function(time,status,alpha=.05){
     out
 }
 
+# create kmciLR objects from kmgw.calc output
+create.kmciLR <- function(x, all_times=NULL){
+  # x is output from kmgw.calc
+  
+  ### add values for interval (0, t1): d=0,n=N,S=1,gw=0
+  new_time <- 0
+  new_ci <- 0
+  new_di <- 0
+  new_KM <- 1
+  new_gw <- 0
+  
+  # if all times not provided, just assume x has all times
+  if(is.null(all_times)) {
+    # add in + symbol for censoring time
+    event_times <- unique( x$time[which( x$di>0)])
+    cens_times <- unique( x$time[which( x$ci>0)])
+    sorting_i <- sort(c(event_times,cens_times), index.return=T)
+    
+    all_times <- c(event_times,paste0(cens_times,"+"))[sorting_i$ix]
+  } 
+  
+    for(t_char in all_times){
+      #t_num = as.numeric(stringi::stri_replace_all_fixed(str=t_char, pattern="+",replacement = ""))
+      t_num = as.numeric(gsub(x=t_char, pattern="+",replacement = "", fixed=TRUE))
+    # the function last is not part of base R
+    last<-function(x){ tail(x,1) }
+    
+    # check if this time is in the x data 
+    if(t_num %in% x$time) {
+      x_i <- which(x$time==t_num)
+      
+      # is this an event time or a censoring time?
+      #cens_i <- stringi::stri_detect_fixed(str=t_char, pattern="+")
+      cens_i <- grepl(x=t_char, pattern="+",fixed = TRUE)
+      
+    if(!cens_i){
+      # add new event time
+      new_time <- c(new_time,t_num)
+      
+      # update new events, KM, and gw
+      new_di <- c(new_di,x$di[x_i])
+      new_KM <-  c(new_KM,x$KM[x_i])
+      new_gw <- c(new_gw,x$gw[x_i])
+      
+      # no new censoring
+      new_ci <- c(new_ci, 0)
+      
+      
+    } else{
+      # add new censoring time
+      new_time <- c(new_time,t_num)
+      
+      # No new events
+      new_di <- c(new_di,0)
+      
+      # KM and gw stay the same from last
+      new_KM <-  c(new_KM,last(new_KM))
+      new_gw <- c(new_gw,last(new_gw))
+      
+      # update new censoring
+      new_ci <- c(new_ci, x$ci[x_i])
+    
+    
+    } 
+      } else {
+      # no new event or censoring time, keep everything the same from the last
+      new_time <- c(new_time,t_num)
+      new_di <- c(new_di,0)
+      new_KM <-  c(new_KM,last(new_KM))
+      new_gw <- c(new_gw,last(new_gw))
+      new_ci <- c(new_ci, 0)
+    }
+  }
+  
+  # recalculate number at risk
+  new_ni <- c(x$ni[1], x$ni[1] - cumsum(new_ci)[-length(new_ci)] - cumsum(new_di)[-length(new_di)])
+  
+  # create left and right intervals 
+  L <- new_time
+  R <- c(new_time[-1], Inf)
+  
+  # create inclusion based on censoring
+  #Lin <- c(TRUE,!stringi::stri_detect_fixed(str=all_times, pattern="+"))
+  Lin <- c(TRUE,!grepl(x=all_times, pattern="+",fixed = TRUE))
+  Rin <- rep(TRUE, length(Lin))
+  Rin[which(Lin)-1] <- FALSE
+  
+ 
+  out <- list(time=new_time,
+              L=L,
+              Lin=Lin,
+              R=R,
+              Rin=Rin,
+              ci=new_ci,
+              di=new_di,
+              ni=new_ni,
+              KM=new_KM,
+              gw=new_gw)
+  
+  return(out)
+}
+
+
+# this function runs borkowf's methods on a single sample
+# which can then be used in either one-sample or two-sample case
+# returns KM estimate, shrunken KM estimate, and 3 variance estimates (greenwood, regular hybrid, adjusted hybrid)
+# all_times is used by the two-sample case so that intervals are based on times of all groups in the data, not just individual group
+delta.calc <- function(time_g,status_g, all_times=NULL,trans, method, alpha,zero.one.adjustment,two_samp_out){
+
+  # Calculate KM estimate and greenwood variance for each time
+  x<-kmgw.calc(time_g,status_g,keepCens=T)
+  
+  # expand times and create intervals based on unique events/censoring times
+  delta_x <- create.kmciLR(x, all_times=all_times)
+     
+  # run borkowf calculations
+  
+  d <- delta_x$di
+  c <- delta_x$ci
+  me<-cumsum(d)
+  mc<-cumsum(c)
+  
+  n<-delta_x$ni[1]
+  bmax<- 1 - me/n
+  w<-rep(NA,length(d))
+  
+  ## KM estimate
+  KM<-delta_x$KM
+  w[.5<=KM]<- KM[.5<=KM]
+  w[KM<.5 & .5<=bmax]<- .5
+  w[bmax<.5]<-bmax[bmax<.5]
+  
+  ## regular hybrid variance estimator
+  # n-mc is the effective sample size at a particular time.
+  VarH<- w*(1-w)/(n - mc)
+  
+  ## shrunken KM
+  KMs<- KM*(1- 1/n) + 1/(2*n)
+  
+  ## adjusted hybrid variance estimator
+  ws<- w*(1- 1/n) + 1/(2*n) 
+  VarHs<- ws*(1-ws)/(n - mc)
+  
+  # recalculate variance estimators
+  # If all of the subjects are censored, set the sample size to a nominal value of 0.5 when all subjects become censored
+  if(sum(d)==0){
+    # find last censoring time
+    last.c <- which(c>0)[length(which(c>0))]
+    
+    VarH[(last.c):length(VarH)] <- w[(last.c):length(VarH)]*(1-w[(last.c):length(VarH)])/0.5
+    VarHs[(last.c):length(VarH)] <- ws[(last.c):length(VarH)]*(1-ws[(last.c):length(VarH)])/0.5
+  }
+  
+  
+  # transformation functions
+  h <- get_h(trans=trans)
+  dh <- get_dh(trans=trans)
+  hinv <- get_hinv(trans=trans)
+  
+  # calculate Surv and Sigma by method 
+  
+  Surv <- switch(method,
+                 standard=KM,
+                 reg_hybrid=KM,
+                 adj_hybrid= KM,
+                 sh_adj_hybrid=KMs)
+  
+  Sigma <- switch(method,
+                  standard=calc_sigma(S=KM, V=delta_x$gw, dh=dh),
+                  reg_hybrid=calc_sigma(S=KM, V=VarH, dh=dh),
+                  adj_hybrid= calc_sigma(S=KM, V=VarHs, dh=dh),
+                  sh_adj_hybrid=calc_sigma(S=KMs, V=VarHs, dh=dh))
+  
+  
+  # base direction of CI on transformation
+  phi <- ifelse(trans=="clog" | trans=="loglog", -1, 1)
+  Za<-qnorm(1-alpha/2)
+  
+  # calculate upper and lower CIs
+  h.ci.lo <- h(Surv) - phi*Za*sqrt(Sigma)
+  h.ci.up <- h(Surv) + phi*Za*sqrt(Sigma)
+  
+  if(zero.one.adjustment){
+    
+    # S=0, lower limit = 0
+    # S=1, upper limit = 1
+    h.ci.lo[which(Surv==0)] <- h(0)
+    h.ci.up[which(Surv==1)] <- h(1)
+    
+    # calculate Vdot based on method
+    if(method=="standard"){
+      Vdot <- KMs*(1-KMs)/delta_x$ni
+      
+    } else{
+      Vdot <- VarHs
+      
+    }
+    # adjust upper limit for S=0
+    h.ci.up[which(Surv==0)] <- (h(KMs) + phi*Za*sqrt(calc_sigma(KMs,Vdot, dh)))[which(Surv==0)]
+    
+    #adjust lower limit for S=1
+    h.ci.lo[which(Surv==1)] <- (h(KMs) - phi*Za*sqrt(calc_sigma(KMs,Vdot, dh)))[which(Surv==1)]
+    
+  } 
+  
+  
+  # reverse transform CI
+  hinv.ci.lo=hinv(h.ci.lo)
+  hinv.ci.up=hinv(h.ci.up)
+  
+  # adjust CI so that it is between 0 and 1
+  hinv.ci.lo.adj=ifelse(hinv.ci.lo<0,0,hinv.ci.lo)
+  hinv.ci.up.adj=ifelse(hinv.ci.up>1,1,hinv.ci.up)
+  
+  # for one-sample case, return kmci output
+  if(!two_samp_out){
+  
+  out <- list(cens=delta_x$time[which(c>0)],L=delta_x$L,Lin=delta_x$Lin,R=delta_x$R, Rin=delta_x$Rin,
+  surv=hinv(h(Surv)),lower=hinv.ci.lo.adj,upper=hinv.ci.up.adj,conf.level=1-alpha)
+
+  } else{
+    
+    # in two-sample case, return KM and variance estimators + upper and lower CI
+    
+    out <- list(cens=delta_x$time[which(c>0)], ni=delta_x$ni,L=delta_x$L,Lin=delta_x$Lin,R=delta_x$R, Rin=delta_x$Rin,
+                KM=KM, gw=delta_x$gw, VarH= VarH,  KMs=KMs, VarHs=VarHs,surv=hinv(h(Surv)),lower=hinv.ci.lo.adj,upper=hinv.ci.up.adj)
+  }
+ 
+  
+  
+  
+  
+  out
+}
+
+# function to calculate sigma
+calc_sigma <- function(S, V, dh){ 
+  Sigma <- rep(NA, length(S))
+  # Will always be 0 if V = 0 (hard coding in case dh(S)=Inf or -Inf)
+  Sigma[which(V==0)] <- 0
+  Sigma[which(V!=0)] <- dh(S[which(V!=0)])^2*V[which(V!=0)]
+  
+  Sigma
+}
+
+# this function is for the one-sample case, will call delta.calc and calculate appropriate CI
+kmciDelta <- function(time,status,alpha=.05, 
+                      trans=c("none","log","cloglog","logodds","clog","loglog"), 
+                      zero.one.adjustment=FALSE,
+                      method=c("standard","reg_hybrid","adj_hybrid","sh_adj_hybrid"),
+                      all_times=NULL){
+  
+  trans<- match.arg(trans)
+  method<- match.arg(method)
+  
+  ## argument checking
+  if (length(time)!=length(status)) stop("length of time and group should be the same")
+  
+  # get estimates for survival and variance
+  out <- delta.calc(time_g=time,status_g=status, all_times = all_times, trans=trans, method=method, alpha=alpha,zero.one.adjustment=zero.one.adjustment,two_samp_out=FALSE)
+
+  # return KMCI object
+  out$num <- length(time)
+  out$events <- length(which(status == max(status)))
+  
+  class(out)<-"kmciLR"
+  
+  
+  return(out)
+}
+
 
 kmConstrain<-function(tstar,pstar,x,alpha=.05){
     ## KM given that S(t)=pstar
@@ -863,11 +1145,11 @@ plot.kmciLR<-function(x,XLAB="time",YLAB="Survival",
 plot.kmciLRtidy <- function(x, ...) {
   tidyout <- tidykmciLR(x)
   if (ncol(tidyout) == 5) {
-    ggplot(tidyout, aes_string(x = "time", y = "surv", ymin = "lower", ymax = "upper", linetype = "group")) + 
+    ggplot(tidyout, aes(x = tidyout$time, y = tidyout$surv, ymin = tidyout$lower, ymax = tidyout$upper, linetype = tidyout$group)) + 
       geom_line() + geom_ribbon(alpha = .2) + labs(linetype = x[[1]]$groupVarName) + xlab("Time") + ylab("Survival")
   }
   else {
-    ggplot(tidyout, aes_string(x = "time", y = "surv", ymin = "lower", ymax = "upper")) + 
+    ggplot(tidyout, aes(x = tidyout$time, y = tidyout$surv, ymin = tidyout$lower, ymax = tidyout$upper)) + 
       geom_line() + geom_ribbon(alpha = .2) + xlab("Time") + ylab("Survival")
   }
 }
@@ -1106,11 +1388,15 @@ StCI.default<-function(x,tstar,afterMax="continue",...){
 StCI.kmciLR<-function(x,tstar,...){
     ### get survival and confidence interval at t 
     ### from kmciLR object
+  
+  # if only one treatment variable, unlist
+  if(is.null(x$surv)){x <- x[[1]]}
     nt<-length(tstar)
     I<-rep(NA,nt)
     index<-1:length(x$surv)
     ## picki gives TRUE/FALSE vector, TRUE where tval fits 
     ## into interval
+    
     picki<-function(tval){
         (x$L<tval & x$R>tval) | (x$L==tval & x$Lin) | 
         (x$R==tval & x$Rin)
@@ -1132,6 +1418,39 @@ StCI.kmciLR<-function(x,tstar,...){
     out     
 }
 
+StCI.kmciLRtidy<-function(x,tstar,...){
+  ### get survival and confidence interval at t 
+  ### from kmciLR object
+  
+  # if only one treatment variable, make sure list isn't nested
+  if(is.null(x$surv)){x <- x[[1]]}
+  nt<-length(tstar)
+  I<-rep(NA,nt)
+  index<-1:length(x$surv)
+  ## picki gives TRUE/FALSE vector, TRUE where tval fits 
+  ## into interval
+  
+  picki<-function(tval){
+    (x$L<tval & x$R>tval) | (x$L==tval & x$Lin) | 
+      (x$R==tval & x$Rin)
+  }
+  for (j in 1:nt){
+    I[j]<-index[picki(tstar[j])]
+  }
+  out<-data.frame(time=tstar,survival=x$surv[I],
+                  lower=x$lower[I],upper=x$upper[I])
+  ## changing the name of the column in the data.frame 
+  ## after the conf.level, was a bad idea.
+  ## It is cleaner to add conf.level as an attribute. 
+  #dimnames(out)[[2]]<-c("time","survival",
+  #       paste("lower ",100*x$conf.level,"% CL",sep=""),
+  #       paste("upper ",100*x$conf.level,"% CL",sep=""))
+  #print(out,row.names=FALSE)
+  attr(out,"conf.level")<- x$conf.level
+  #invisible(out)
+  out     
+}
+
 quantile.kmciLR<-function(x,probs=c(.25,.5,.75),...){
     lower<-upper<-surv<-rep(NA,length(probs))
 
@@ -1463,6 +1782,16 @@ bpcp.mc<-function(x,nmc=100,alpha=.05, testtime=0, DELTA=0, midp=FALSE){
 }
 
 
+statusCheck<-function(status){
+  stopMessage<- "status must be a vector with values 0 or 1"
+  if (!(is.numeric(status) | is.integer(status))) stop(stopMessage)
+  ustatus<- sort(unique(status))
+  if (length(ustatus)>2){ stop(stopMessage)
+  } else if (length(ustatus)==2){
+    if (!all(ustatus==c(0,1))) stop(stopMessage)
+  } else if (!(ustatus==1 | ustatus==0)) stop(stopMessage)
+}
+
 
 
 
@@ -1476,6 +1805,11 @@ bpcp<-function(time,status,nmc=0,alpha=.05,Delta=0,stype="km", midp=FALSE, monot
             monotonic<-FALSE
         }
     }
+  
+    if(is.null(status)){
+      status = rep(1,length(time))
+    }
+    statusCheck(status)
     midpMMTol<-control$midpMMTol
     # so we get the same answer with the same data set, 
     # default to set seed...control$seed=NULL is for
@@ -1584,12 +1918,15 @@ bpcp<-function(time,status,nmc=0,alpha=.05,Delta=0,stype="km", midp=FALSE, monot
         lower<-cummin(lower)
         upper<-cummin(upper)
     }
+    num <- length(time)
+    events <- length(which(status == max(status)))
     out<-list(cens=getmarks(time,status),surv=SURV,
         lower=lower,
         upper=upper,
         L=L[keep],Lin=Lin[keep],R=R[keep],Rin=Rin[keep],
         Interval=intChar(L,R,Lin,Rin)[keep],stype=stype, 
-        betaParms=betaParms, conf.level=1-alpha)
+        betaParms=betaParms, conf.level=1-alpha, num=num, events=events)
+
     class(out)<-"kmciLR"
     out
 }
@@ -1636,11 +1973,18 @@ bpcpfit.formula <- function(formula, data, subset, na.action, ...) {
   if (indx[1]==0) {
     stop("a formula argument is required")
   }
+
   if (missing(data)) 
     data <- environment(formula)
-  #Change to model.frame format using data
-  temp <- model.frame(formula, data=data)
-  #Evaluate
+  #Change to model.frame format using data and check for warning message
+  tc <- tryCatch(model.frame(formula, data=data),
+           warning=function(w) w,
+           error=function(e) e)
+  if(is(tc,"warning")){if(tc$message=="Invalid status value, converted to NA") stop("status must be a vector with values 0 or 1")}
+  if(is(tc,"error")) stop("Response must be a survival (Surv) object")
+  
+  temp <-   model.frame(formula, data=data)
+#Evaluate
   mf <- eval.parent(temp)
   #Get terms of formula
   Terms <- terms(formula)
@@ -1656,6 +2000,31 @@ bpcpfit.formula <- function(formula, data, subset, na.action, ...) {
   if (!is.Surv(Y)) {
     stop("Response must be a survival (Surv) object")
   }
+  #Make sure status follows correct formatting if provided
+  if(length(terms(formula)[[2]])==3 ) {
+    
+    if(terms(formula)[[2]][[3]]=="status"){
+      
+      status <- data[["status"]]
+      
+    } else{
+      status <- terms(formula)[[2]][[3]]
+    }
+    
+    statusCheck(status)
+    
+  } else if (length(terms(formula)[[2]])==2 & "status" %in% names(terms(formula)[[2]])){
+    
+    if(terms(formula)[[2]][[2]]=="status"){
+      
+      status <- data[["status"]]
+      
+    } else{
+      status <- terms(formula)[[2]][[2]]
+    }
+    
+    statusCheck(status)
+  }
   #Get labels from formula (group variable)
   ll <- attr(Terms, 'term.labels')
   if (length(ll) > 1) {
@@ -1693,16 +2062,21 @@ bpcpfit.default <- function(time, status = NULL, group = NULL, formula=NULL, nmc
     stop("Time is required.")
   }
   if (!is.null(alpha)) {
-    print("Warning: alpha is out of date. Use conf.level instead. Setting conf.level to 1-alpha and ignoring conf.level.")
+    warning("Warning: alpha is out of date. Use conf.level instead. Setting conf.level to 1-alpha and ignoring conf.level.")
   }
   else {
     alpha <- 1-conf.level
   }
   Call <- match.call()
+  
+  if(is.null(status)){
+    status <- rep(1,length(time))
+  }
+  
+  statusCheck(status)
+  
   if (is.null(group)) {
-    if (length(unique(status)) > 2) {
-      stop("Interval censoring is not supported by bpcp or bpcpfit.")
-    }
+
     out <- bpcp(time, status, nmc=nmc, alpha=alpha, Delta=Delta, stype=stype, midp=midp,
                 monotonic=monotonic, control=control)
     out$num <- length(time)
@@ -1712,8 +2086,10 @@ bpcpfit.default <- function(time, status = NULL, group = NULL, formula=NULL, nmc
       results <- append(results, list(out))
       class(results) <- "kmciLRtidy"
     }
-    else {
+    else if (plotstyle == "standard") {
       results <- out
+    } else {
+      stop('Plot style must be either "ggplot" or "standard"')
     }
   }
   else {
@@ -1724,9 +2100,7 @@ bpcpfit.default <- function(time, status = NULL, group = NULL, formula=NULL, nmc
     results <- list()
     #Iterate through each level of the treatment variable, and do the bpcp function of the corresponding data. Store the results in a list
     for (i in 1:length(newZ)) {
-      if (length(unique(newZ[[i]]$V2)) > 2) {
-        stop("Interval censoring is not supported by bpcp or bpcpfit.")
-      }
+      
       results <- append(results, list(bpcp(newZ[[i]]$V1, newZ[[i]]$V2, nmc=nmc, alpha=alpha, Delta=Delta, stype=stype, midp=midp,
                                            monotonic=monotonic, control=control)))
       results[[i]]$num <- length(newZ[[i]]$V1)
